@@ -18,13 +18,12 @@
 Soledad-backed IMAP Server.
 """
 import copy
-import email
 import logging
-import re
 import StringIO
 import cStringIO
 import time
 
+from collections import defaultdict
 from email.parser import Parser
 
 from zope.interface import implements
@@ -33,13 +32,10 @@ from twisted.mail import imap4
 from twisted.internet import defer
 from twisted.python import log
 
-#from twisted import cred
-
-#import u1db
-
 from leap.common import events as leap_events
 from leap.common.events.events_pb2 import IMAP_UNREAD_MAIL
 from leap.common.check import leap_assert, leap_assert_type
+from leap.common.mail import get_email_charset
 from leap.soledad.client import Soledad
 
 logger = logging.getLogger(__name__)
@@ -184,7 +180,8 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
         # messages
         TYPE_MBOX_SEEN_IDX: [KTYPE, MBOX_VAL, 'bool(seen)'],
         TYPE_MBOX_RECT_IDX: [KTYPE, MBOX_VAL, 'bool(recent)'],
-        TYPE_MBOX_RECT_SEEN_IDX: [KTYPE, MBOX_VAL, 'bool(recent)', 'bool(seen)'],
+        TYPE_MBOX_RECT_SEEN_IDX: [KTYPE, MBOX_VAL,
+                                  'bool(recent)', 'bool(seen)'],
     }
 
     INBOX_NAME = "INBOX"
@@ -245,6 +242,7 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
 
         :rtype: SoledadDocument
         """
+        # XXX only upper for INBOX ---
         name = name.upper()
         doc = self._soledad.get_from_index(
             self.TYPE_MBOX_IDX, self.MBOX_KEY, name)
@@ -278,6 +276,7 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
         :returns: a a SoledadMailbox instance
         :rtype: SoledadMailbox
         """
+        # XXX only upper for INBOX
         name = name.upper()
         if name not in self.mailboxes:
             raise imap4.MailboxException("No such mailbox")
@@ -303,6 +302,7 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
         :returns: True if successful
         :rtype: bool
         """
+        # XXX only upper for INBOX
         name = name.upper()
         # XXX should check mailbox name for RFC-compliant form
 
@@ -364,6 +364,7 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
 
         :rtype: bool
         """
+        # XXX only upper for INBOX
         name = name.upper()
 
         if name not in self.mailboxes:
@@ -389,6 +390,7 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
                       names. use with care.
         :type force: bool
         """
+        # XXX only upper for INBOX
         name = name.upper()
         if not name in self.mailboxes:
             raise imap4.MailboxException("No such mailbox")
@@ -426,6 +428,7 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
         :param newname: new name of the mailbox
         :type newname: str
         """
+        # XXX only upper for INBOX
         oldname = oldname.upper()
         newname = newname.upper()
 
@@ -491,7 +494,6 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
         # maybe we should store subscriptions in another
         # document...
         if not name in self.mailboxes:
-            print "not this mbox"
             self.addMailbox(name)
         mbox = self._get_mailbox_by_name(name)
 
@@ -577,7 +579,7 @@ class SoledadBackedAccount(WithMsgFields, IndexedDB):
         return "<SoledadBackedAccount (%s)>" % self._account_name
 
 #######################################
-# Soledad Message, MessageCollection
+# LeapMessage, MessageCollection
 # and Mailbox
 #######################################
 
@@ -695,26 +697,6 @@ class LeapMessage(WithMsgFields):
     the more complex MIME-based interface.
     """
 
-    def _get_charset(self, content):
-        """
-        Mini parser to retrieve the charset of an email
-
-        :param content: mail contents
-        :type content: unicode
-
-        :returns: the charset as parsed from the contents
-        :rtype: str
-        """
-        charset = "UTF-8"
-        try:
-            em = email.message_from_string(content.encode("utf-8"))
-            # Miniparser for: Content-Type: <something>; charset=<charset>
-            charset_re = r'''charset=(?P<charset>[\w|\d|-]*)'''
-            charset = re.findall(charset_re, em["Content-Type"])[0]
-        except Exception:
-            pass
-        return charset
-
     def open(self):
         """
         Return an file-like object opened for reading.
@@ -726,8 +708,14 @@ class LeapMessage(WithMsgFields):
         :rtype: StringIO
         """
         fd = cStringIO.StringIO()
-        charset = self._get_charset(self._doc.content.get(self.RAW_KEY, ''))
-        fd.write(self._doc.content.get(self.RAW_KEY, '').encode(charset))
+        charset = get_email_charset(self._doc.content.get(self.RAW_KEY, ''))
+        content = self._doc.content.get(self.RAW_KEY, '')
+        try:
+            content = content.encode(charset)
+        except (UnicodeEncodeError, UnicodeDecodeError) as e:
+            logger.error("Unicode error {0}".format(e))
+            content = content.encode(charset, 'replace')
+        fd.write(content)
         fd.seek(0)
         return fd
 
@@ -746,8 +734,14 @@ class LeapMessage(WithMsgFields):
         :rtype: StringIO
         """
         fd = StringIO.StringIO()
-        charset = self._get_charset(self._doc.content.get(self.RAW_KEY, ''))
-        fd.write(self._doc.content.get(self.RAW_KEY, '').encode(charset))
+        charset = get_email_charset(self._doc.content.get(self.RAW_KEY, ''))
+        content = self._doc.content.get(self.RAW_KEY, '')
+        try:
+            content = content.encode(charset)
+        except (UnicodeEncodeError, UnicodeDecodeError) as e:
+            logger.error("Unicode error {0}".format(e))
+            content = content.encode(charset, 'replace')
+        fd.write(content)
         # SHOULD use a separate BODY FIELD ...
         fd.seek(0)
         return fd
@@ -797,6 +791,7 @@ class LeapMessage(WithMsgFields):
         return dict(filter_by_cond)
 
     # --- no multipart for now
+    # XXX Fix MULTIPART SUPPORT!
 
     def isMultipart(self):
         return False
@@ -979,6 +974,7 @@ class MessageCollection(WithMsgFields, IndexedDB):
         docs = self._soledad.get_from_index(
             SoledadBackedAccount.TYPE_MBOX_UID_IDX,
             self.TYPE_MESSAGE_VAL, self.mbox, str(uid))
+
         return docs[0] if docs else None
 
     def get_msg_by_uid(self, uid):
@@ -996,6 +992,47 @@ class MessageCollection(WithMsgFields, IndexedDB):
         if doc:
             return LeapMessage(doc)
 
+    def get_by_index(self, index):
+        """
+        Retrieves a mesage document by mailbox index.
+
+        :param index: the index of the sequence (zero-indexed)
+        :type index: int
+        """
+        try:
+            return self.get_all()[index]
+        except IndexError:
+            return None
+
+    def get_msg_by_index(self, index):
+        """
+        Retrieves a LeapMessage by sequence index.
+
+        :param index: the index of the sequence (zero-indexed)
+        :type index: int
+        """
+        doc = self.get_by_index(index)
+        if doc:
+            return LeapMessage(doc)
+
+    def is_deleted(self, doc):
+        """
+        Returns whether a given doc is deleted or not.
+
+        :param doc: the document to check
+        :rtype: bool
+        """
+        return self.DELETED_FLAG in doc.content[self.FLAGS_KEY]
+
+    def get_last(self):
+        """
+        Gets the last LeapMessage
+        """
+        _all = self.get_all()
+        if not _all:
+            return None
+        return LeapMessage(_all[-1])
+
     def get_all(self):
         """
         Get all message documents for the selected mailbox.
@@ -1005,9 +1042,13 @@ class MessageCollection(WithMsgFields, IndexedDB):
         :rtype: list of SoledadDocument
         """
         # XXX this should return LeapMessage instances
-        return self._soledad.get_from_index(
+        all_docs = [doc for doc in self._soledad.get_from_index(
             SoledadBackedAccount.TYPE_MBOX_IDX,
-            self.TYPE_MESSAGE_VAL, self.mbox)
+            self.TYPE_MESSAGE_VAL, self.mbox)]
+            #if not self.is_deleted(doc)]
+        # highly inneficient, but first let's grok it and then
+        # let's worry about efficiency.
+        return sorted(all_docs, key=lambda item: item.content['uid'])
 
     def unseen_iter(self):
         """
@@ -1087,8 +1128,11 @@ class MessageCollection(WithMsgFields, IndexedDB):
         :return: LeapMessage or None if not found.
         :rtype: LeapMessage
         """
+        #try:
+            #return self.get_msg_by_uid(uid)
         try:
-            return self.get_msg_by_uid(uid)
+            return [doc
+                    for doc in self.get_all()][uid - 1]
         except IndexError:
             return None
 
@@ -1111,6 +1155,7 @@ class SoledadMailbox(WithMsgFields):
     which we instantiate and make accessible in the `messages` attribute.
     """
     implements(imap4.IMailboxInfo, imap4.IMailbox, imap4.ICloseableMailbox)
+    # XXX should finish the implementation of IMailboxListener
 
     messages = None
     _closed = False
@@ -1127,6 +1172,8 @@ class SoledadMailbox(WithMsgFields):
     CMD_UIDVALIDITY = "UIDVALIDITY"
     CMD_UNSEEN = "UNSEEN"
 
+    _listeners = defaultdict(set)
+
     def __init__(self, mbox, soledad=None, rw=1):
         """
         SoledadMailbox constructor. Needs to get passed a name, plus a
@@ -1159,28 +1206,54 @@ class SoledadMailbox(WithMsgFields):
         if not self.getFlags():
             self.setFlags(self.INIT_FLAGS)
 
-        # XXX what is/was this used for? --------
-        # ---> mail/imap4.py +1155,
-        #      _cbSelectWork makes use of this
-        # probably should implement hooks here
-        # using leap.common.events
-        self.listeners = []
-        self.addListener = self.listeners.append
-        self.removeListener = self.listeners.remove
-        #------------------------------------------
+    @property
+    def listeners(self):
+        """
+        Returns listeners for this mbox.
+
+        The server itself is a listener to the mailbox.
+        so we can notify it (and should!) after changes in flags
+        and number of messages.
+
+        :rtype: set
+        """
+        return self._listeners[self.mbox]
+
+    def addListener(self, listener):
+        """
+        Rdds a listener to the listeners queue.
+
+        :param listener: listener to add
+        :type listener: an object that implements IMailboxListener
+        """
+        logger.debug('adding mailbox listener: %s' % listener)
+        self.listeners.add(listener)
+
+    def removeListener(self, listener):
+        """
+        Removes a listener from the listeners queue.
+
+        :param listener: listener to remove
+        :type listener: an object that implements IMailboxListener
+        """
+        self.listeners.remove(listener)
 
     def _get_mbox(self):
         """
         Returns mailbox document.
 
-        :return: A SoledadDocument containing this mailbox.
-        :rtype: SoledadDocument
+        :return: A SoledadDocument containing this mailbox, or None if
+                 the query failed.
+        :rtype: SoledadDocument or None.
         """
-        query = self._soledad.get_from_index(
-            SoledadBackedAccount.TYPE_MBOX_IDX,
-            self.TYPE_MBOX_VAL, self.mbox)
-        if query:
-            return query.pop()
+        try:
+            query = self._soledad.get_from_index(
+                SoledadBackedAccount.TYPE_MBOX_IDX,
+                self.TYPE_MBOX_VAL, self.mbox)
+            if query:
+                return query.pop()
+        except Exception as exc:
+            logger.error("Unhandled error %r" % exc)
 
     def getFlags(self):
         """
@@ -1192,6 +1265,7 @@ class SoledadMailbox(WithMsgFields):
         #return map(str, self.INIT_FLAGS)
 
         # XXX CHECK against thunderbird XXX
+        # XXX I think this is slightly broken.. :/
 
         mbox = self._get_mbox()
         if not mbox:
@@ -1276,8 +1350,12 @@ class SoledadMailbox(WithMsgFields):
 
         :rtype: int
         """
-        # XXX reimplement with proper index
-        return self.messages.count() + 1
+        last = self.messages.get_last()
+        if last:
+            nextuid = last.getUID() + 1
+        else:
+            nextuid = 1
+        return nextuid
 
     def getMessageCount(self):
         """
@@ -1364,6 +1442,12 @@ class SoledadMailbox(WithMsgFields):
 
         self.messages.add_msg(message, flags=flags, date=date,
                               uid=uid_next)
+
+        # XXX recent should not include deleted...??
+        exists = len(self.messages)
+        recent = len(self.messages.get_recent())
+        for listener in self.listeners:
+            listener.newMessages(exists, recent)
         return defer.succeed(None)
 
     # commands, do not rename methods
@@ -1419,16 +1503,35 @@ class SoledadMailbox(WithMsgFields):
         :rtype: A tuple of two-tuples of message sequence numbers and
                 LeapMessage
         """
-        # XXX implement sequence numbers (uid = 0)
         result = []
+        sequence = True if uid == 0 else False
 
         if not messages.last:
-            messages.last = self.messages.count()
+            try:
+                iter(messages)
+            except TypeError:
+                # looks like we cannot iterate
+                last = self.messages.get_last()
+                uid_last = last.getUID()
+                messages.last = uid_last
+
+        # for sequence numbers (uid = 0)
+        if sequence:
+            for msg_id in messages:
+                msg = self.messages.get_msg_by_index(msg_id - 1)
+                if msg:
+                    result.append((msg.getUID(), msg))
+                else:
+                    print "fetch %s, no msg found!!!" % msg_id
+
+        else:
+            for msg_id in messages:
+                msg = self.messages.get_msg_by_uid(msg_id)
+                if msg:
+                    result.append((msg_id, msg))
+                else:
+                    print "fetch %s, no msg found!!!" % msg_id
 
-        for msg_id in messages:
-            msg = self.messages.get_msg_by_uid(msg_id)
-            if msg:
-                result.append((msg_id, msg))
         return tuple(result)
 
     def _signal_unread_to_ui(self):
