@@ -16,7 +16,7 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """
-SMTP relay helper function.
+SMTP gateway helper function.
 """
 import logging
 
@@ -26,66 +26,49 @@ from twisted.internet.error import CannotListenError
 logger = logging.getLogger(__name__)
 
 from leap.common.events import proto, signal
-from leap.mail.smtp.smtprelay import SMTPFactory
+from leap.mail.smtp.gateway import SMTPFactory
 
 
-def setup_smtp_relay(port, keymanager, smtp_host, smtp_port,
+def setup_smtp_gateway(port, userid, keymanager, smtp_host, smtp_port,
                      smtp_cert, smtp_key, encrypted_only):
     """
-    Setup SMTP relay to run with Twisted.
+    Setup SMTP gateway to run with Twisted.
 
-    This function sets up the SMTP relay configuration and the Twisted
+    This function sets up the SMTP gateway configuration and the Twisted
     reactor.
 
     :param port: The port in which to run the server.
     :type port: int
+    :param userid: The user currently logged in
+    :type userid: unicode
     :param keymanager: A Key Manager from where to get recipients' public
                        keys.
     :type keymanager: leap.common.keymanager.KeyManager
     :param smtp_host: The hostname of the remote SMTP server.
     :type smtp_host: str
-    :param smtp_port:  The port of the remote SMTP server.
+    :param smtp_port: The port of the remote SMTP server.
     :type smtp_port: int
     :param smtp_cert: The client certificate for authentication.
     :type smtp_cert: str
     :param smtp_key: The client key for authentication.
     :type smtp_key: str
-    :param encrypted_only: Whether the SMTP relay should send unencrypted mail
+    :param encrypted_only: Whether the SMTP gateway should send unencrypted mail
                            or not.
     :type encrypted_only: bool
 
     :returns: tuple of SMTPFactory, twisted.internet.tcp.Port
     """
-    # The configuration for the SMTP relay is a dict with the following
-    # format:
-    #
-    # {
-    #     'host': '<host>',
-    #     'port': <int>,
-    #     'cert': '<cert path>',
-    #     'key': '<key path>',
-    #     'encrypted_only': <True/False>
-    # }
-    config = {
-        'host': smtp_host,
-        'port': smtp_port,
-        'cert': smtp_cert,
-        'key': smtp_key,
-        'encrypted_only': encrypted_only
-    }
-
     # configure the use of this service with twistd
-    factory = SMTPFactory(keymanager, config)
+    factory = SMTPFactory(userid, keymanager, smtp_host, smtp_port, smtp_cert,
+                          smtp_key, encrypted_only)
     try:
-        tport = reactor.listenTCP(port, factory,
-                                  interface="localhost")
-        signal(proto.SMTP_SERVICE_STARTED, str(smtp_port))
+        tport = reactor.listenTCP(port, factory, interface="localhost")
+        signal(proto.SMTP_SERVICE_STARTED, str(port))
         return factory, tport
     except CannotListenError:
         logger.error("STMP Service failed to start: "
-                     "cannot listen in port %s" % (
-                         smtp_port,))
-        signal(proto.SMTP_SERVICE_FAILED_TO_START, str(smtp_port))
+                     "cannot listen in port %s" % port)
+        signal(proto.SMTP_SERVICE_FAILED_TO_START, str(port))
     except Exception as exc:
-        logger.error("Unhandled error while launching smtp relay service")
+        logger.error("Unhandled error while launching smtp gateway service")
         logger.exception(exc)
